#include <Wire.h>     //For I2C
#include "TimerOne.h" //Get TimerOne from http://code.google.com/p/arduino-timerone/downloads/list
                      // And see example at http://www.arduino.cc/playground/Code/Timer1
#include "pitches.h"  //For making melodies and sound effects with the Buzzer Twig

//***********************************************************************************
//*              88888888ba                                        88888888ba   
//*              88      "8b                                       88      "8b  
//*              88      ,8P                                       88      ,8P  
//*  ,adPPYb,d8  88aaaaaa8P'  ,adPPYba,   8b       d8   ,adPPYba,  88aaaaaa8P'  
//* a8"    `Y88  88""""88'   a8"     "8a  `8b     d8'  a8P_____88  88""""88'    
//* 8b       88  88    `8b   8b       d8   `8b   d8'   8PP"""""""  88    `8b    
//* "8a,   ,d88  88     `8b  "8a,   ,a8"    `8b,d8'    "8b,   ,aa  88     `8b   
//*  `"YbbdP"Y8  88      `8b  `"YbbdP"'       "8"       `"Ybbd8"'  88      `8b  
//*  aa,    ,88                                                                 
//*   "Y8bbdP"   gRoveR, a SeeedStudio Grove Toy Kit Contest Entry, 25-Aug-2011
//*              -v1.0---------------------------------------------------------
//*                      Matthew Lange, http://erroraccessdenied.com    
//*                      -------------------------------------------
//*
//*  ** Latest code and documentation available at http://erroraccessdenied.com **
//*
//*  Sources: 
//*  --------
//*
//*  * Accelerometer -------(Freescale MMA7660)    
//*      WIKI:         http://seeedstudio.com/wiki/Twig_-_I2C_3-axis_Accelerometer
//*
//*  * Buzzer --------------
//*      Code Example: http://arduino.cc/en/Tutorial/Tone
//*
//*  * I2C Touch Sensor ----(Freescale MPR121)
//*      WIKI:         http://seeedstudio.com/wiki/index.php?title=Twig_-_I2C_Touch_Sensor_v0.93b
//*      Code Example: http://www.prizepony.us/bin/arduino_mpr121.zip
//*
//*  * 128x64 OLED Display -(Solomon SSD1308 (Controller)) and (Liyuan LY190-128064 (OLED))
//*      WIKI:         http://seeedstudio.com/wiki/Twig_-_OLED_Display_128*64
//*      Code Example: http://www.seeedstudio.com/depot/datasheet/oled12864code.zip
//*
//*  * I2C Motor Driver ----(STMicroelectronics L298) and (Atmel ATMega8)
//*      WIKI:         http://seeedstudio.com/wiki/index.php?title=Twig_-_I2C_Motor_Driver
//*
//*  * Chainable RGB LED ---(??? P9813)
//*      WIKI:         http://seeedstudio.com/wiki/index.php?title=Twig_-_Chainable_RGB_LED
//*
//*  * IR Proximity Sensor -(Sharp GP2Y0A21YK)
//*      WIKI:         http://seeedstudio.com/wiki/index.php?title=Twig_-_80cm_Infrared_Proximity_Sensor_v0.9
//*
//*  * PIR Motion Sensor ---(BluNet Intl. RE200B)
//*      WIKI:         http://seeedstudio.com/wiki/Twig_-_PIR_Motion_Sensor
//*
//*  * Vibrator ------------()
//*      WIKI:         http://seeedstudio.com/wiki/Twig_-_Vibrator
//*
//*  * Sound Sensor --------(LM386)
//*      WIKI:         http://seeedstudio.com/wiki/Twig_-_Sound_Sensor
//*
//*  * Sound Recorder ------(APlus Inc. APR9600)
//*      WIKI:         http://seeedstudio.com/wiki/Twig_-_Sound_Recorder_v0.92b
//*
//***********************************************************************************

//*******************
//*  Defines        *
//*******************

// ** (I2C) 3-Axis Accelerometer **
#define MMA7660AccelAddr   0x4C //The Accelerometer I2C address
#define MMA7660Accel_X     0x00 // Read Accelerometer 6-bit X-Axis value of +31 to -32 [Read Only] (pg. 14)
                                //   If Alert Bit is set ( 0bx1xxxxxx ), re-read the values
#define MMA7660Accel_Y     0x01 // Read Accelerometer 6-bit Y-Axis value of +31 to -32 [Read Only] (pg. 14)
                                //   If Alert Bit is set ( 0bx1xxxxxx ), re-read the values
#define MMA7660Accel_Z     0x02 // Read Accelerometer 6-bit Z-Axis value of +31 to -32 [Read Only] (pg. 14)
                                //   If Alert Bit is set ( 0bx1xxxxxx ), re-read the values
#define MMA7660Accel_TILT  0x03 // Read "Tilt Status" [Read Only]  (pg. 15)
#define MMA7660Accel_SRST  0x04 // Read "Sample Rate Status" Register [Read Only]  (pg. 15)
#define MMA7660Accel_SPCNT 0x05 // Read/Set "Sleep Count" Register [Read/Write]  (pg. 15) 
#define MMA7660Accel_INTSU 0x06 // Set "Interrupt Setup" Register (what does/doesn't cause an interrupt) [Write] (pg. 16)
#define MMA7660Accel_MODE  0x07 // Read/Set "Mode" Register [Read/Write] (pg. 17)
#define MMA7660Accel_SR    0x08 // Read/Set "Auto-Wake and Active Mode Portrait/Landscape Samples per Second" Register [Read/Write]  (pg. 18)
#define MMA7660Accel_PDET  0x09 // Read/Set "Tap/Pulse Detection" Register [Read/Write]  (pg. 21)
#define MMA7660Accel_PD    0x0A // Read/Set "Tap/Pulse Debounce" Register [Read/Write[   (pg. 21)
#define MMA7660Accel_IRQ      3 // Interrupt on Digital Pin 3

volatile int accelChanged = 0;  //Has the interrupt sensor changed? This gets set by the interrupt, and we check it when we have time

// ** (I2C) Motor Driver **
#define i2cMotorDriverAddr     0x28
//User Register Addresses.                                                   __________Argument 1____________|__________Argument 2__________
#define i2cMotorDriver_SETPWM  0x82 // *Set PWM for motors A/B.             |  Motor A Speed       [Byte]    |  Motor B Speed        [Byte] |
#define i2cMotorDriver_SETFRQ  0x84 // *Set Frequency                       |  Prescaler           [Byte]    |  N/A                         |
#define i2cMotorDriver_CHADDR  0x83 // *Change I2C Address                  |  New Address         [Byte]    |  Saved (S) or Not (N) [Char] |
#define i2cMotorDriver_CHASET  0xAA // *Channel Set: Set H-Bridge IN1-IN4   |  0 0 0 0 I4 I3 I2 I1 [Byte]    |  N/A                         |
#define i2cMotorDriver_M1SPD   0xA1 // *Motor 1 Speed                       |  "1" Forward / "2" Backward    |  Motor 1 Speed        [Byte] |
#define i2cMotorDriver_M2SPD   0xA5 // *Motor 2 Speed                       |  "1" Forward / "2" Backward    |  Motor 2 Speed        [Byte] |

#define MOTOR_1              0xA1
#define MOTOR_2              0xA5

//#define MOTOR_FWD            0x31//0b01
//#define MOTOR_REV            0x30//0b00
#define MOTOR_FWD            0b01
#define MOTOR_REV            0b10
#define MOTOR_OFF            0b00

// ** (I2C) Touch Sensor **
#define i2cTouchSensor_IRQ           2    //Interrupt on Digital Pin 2
#define i2cTouchSensorAddr           0x5A //The ADDR pin is set to GND, so the address is 0x5A
#define i2cTouchSensor_TOUCH_THRESH  0x0F //Threshold of sensitivity for PRESS
#define i2cTouchSensor_REL_THRESH    0x0A //Threshold of sensitivity for RELEASE
volatile int touchChanged = 0;            //Has the touch sensor changed? This gets set by the interrupt, and we check it when we have time
char touchStatus = 0;                     //Record which buttons are pressed down (binary)

//     Which buttons stand for what?
#define BUTTON_UP        0
#define BUTTON_DOWN      1
#define BUTTON_OK        2
#define BUTTON_CANCEL    3

// ** (I2C) OLED Display 128x64 **
#define oledAddr               0x3c
#define oled_COMMAND           0x80
#define oled_DATA              0x40

//TODO: Make fill_string[] a char array array
unsigned char fill_OLED=0x55;
char fill_string1[]="Error: Access";
char fill_string2[]="Denied";
char fill_string3[]="erroraccess";
char fill_string4[]="denied.com";
char fill_string5[]="Loading";

extern unsigned char myFont[][8]; //PROGMEM
extern unsigned char pos_neg_graph[];
//extern unsigned char max_min_graph[];
//extern unsigned char positive_graph[];
//extern unsigned char ip[];

// ** (I2C) Accelerometer **
//      PS: This used to be in a nice "class" in the demo code, however it got 
//      bumped out into individual char variables during debugging... 
//      No other reason, really  (TODO?)
char AccelX;
char AccelY;
char AccelZ;

// ** (Digital) PIR Motion Sensor **
//      I used the Stem port with only one Digital pin on it,
//      since the PIR only uses one (pinout of GND, VCC, NC, SIG)
const int motionSensor = 12; 

// ** (Digital) Vibrator **
//      I used the Stem port with 12 and 13 on it, since the port
//      is the same as the PIR Motion Sensor; and pin 13 would be
//      unconnected for this particular twig.
const int vibratingMotor = 11;
const int vibratingMotorTwitch = 50;  // Length (in ms) for a twitch
const int vibratingMotorBuzz   = 100; // Length (in ms) for a buzz 

// ** (Digital) Chainable RGB LEDs **
const int clockPin = 4;
const int dataPin = 5;

// ** (Digital) Sound Recorder **
//   The sound recorder takes 2 pins: 
const int soundRecorderPin1 = 6;
const int soundRecorderPin2 = 7;

// ** (Digital) Buzzer **
const int buzzerPin = 8;

// ** (Analog) Sound/Noise Sensor **
const int soundSensor = A0;
const int soundSensorThreshold = 400;

// ** (Analog) 80cm Infrared Proximity Sensor **
const int proximitySensor = A2; 
const int proximitySensorThreshold = 400;

//*******************
//*  Variables      *
//*******************
     
char cursorPosition =  0;     //What row is the cursor currently on?
char cursorActive =    0;     //Are we currently displaying a cursor
int numScreenRows =    0;     //For storing the number of (non-title) rows of text we want to display [can be anywhere from 0..(7-titleHeight)]
//int (*callback)(void) = NULL;
int titleHeight =      0;     //For storing the number of rows that the title and its underline take
int shutDownEVERYTHING = 0;   //A variable that shuts down independent loops
char currScreen =      0;     //Which screen are we on? 0 == Main Menu
                         //                        1 == Sensor View
                         //                        2 == Motor Test Screen
                         //                        3 == Auto Mode
                         //                        4 == Remote Control
                         //                        5 == Cat Spooker
                         //                        6 == Cat Taunter                         
volatile char rgbPattern = 0; //What step are we currently on for the RGB LEDs?

//*******************
//*  Classes        *
//*******************

// N/A

//*******************
//*  Init Functions *
//*******************

//**********************************
// MMA7660Accel_init: Initializes the I2C Accelerometer
//
void MMA7660Accel_init(void)
{ 
  //Puts Accelerometer into Inactive Mode for modifying the parameters
  sendI2cData( MMA7660AccelAddr, MMA7660Accel_MODE, 0x00 );  // 0b00000000
  
      //***********
      //Set Samples/Second Active and Auto-Sleep Mode
      // (Exerpted from MMA7660FC Datasheet, Page 19)
      // -----------------------------------------------
      //      D7   D6  D5       D4  D3       D2  D1  D0   
      //  FILT[2 ...1...0]  AWSR[1...0]  AMSR[2...1...0]
      // -----------------------------------------------
      // FILT[2:0]: 000:        Tilt Debounce Filtering is disabled. Portrait/Landscape updated after each reading
      //            001:        2 Samples must match before Portrait/Landscape is updated
      //            010:        3 Samples must match before Portrait/Landscape is updated
      //            011:        4 Samples must match before Portrait/Landscape is updated
      //            100:        5 Samples must match before Portrait/Landscape is updated
      //            101:        6 Samples must match before Portrait/Landscape is updated
      //            110:        7 Samples must match before Portrait/Landscape is updated
      //            111:        8 Samples must match before Portrait/Landscape is updated
      // AWSR[1:0]: 00  (AW32): 32 Samples/Second in Auto-Wake Mode (Averages the results of 32 measurements once every 31.25ms)
      //            01  (AW16): 16 Samples/Second in Auto-Wake Mode (Averages the results of 32 measurements once every 62.5ms)
      //            10  (AW8):  8  Samples/Second in Auto-Wake Mode (Averages the results of 32 measurements once every 125ms)
      //            11  (AW1):  1  Sample/Second  in Auto-Wake Mode (Averages the results of 32 measurements once every 1000ms)
      // AMSR[2:0]: 000 (AMPD): Tap Detection Mode and 120 Samples/Second Active and Auto-Sleep Mode
      //            001 (AM64): 64 Samples/Second in Active and Auto-Sleep Mode
      //            010 (AM32): 32 Samples/Second in Active and Auto-Sleep Mode
      //            011 (AM16): 16 Samples/Second in Active and Auto-Sleep Mode
      //            100 (AM8):  8  Samples/Second in Active and Auto-Sleep Mode
      //            101 (AM4):  4  Samples/Second in Active and Auto-Sleep Mode
      //            110 (AM2):  2  Samples/Second in Active and Auto-Sleep Mode
      //            111 (AM1):  1  Sample/Second  in Active and Auto-Sleep Mode
  
      sendI2cData( MMA7660AccelAddr, MMA7660Accel_SR,    0x00 ); // 0b00000000
      
      //***********
      //Set "Tap/Pulse Detection" Register
      // (Exerpted from MMA7660FC Datasheet, Page 21) 
      // ---------------------------------------
      //  D7   D6   D5       D4  D3  D2  D1  D0   
      //  ZDA  YDA  XDA  PDTH[4...3...2...1...0]
      // ---------------------------------------
      // [Errata: Datasheet describes "PD[4:0]", it should be "PD[7:0]"
      // PD[7:0]: 00000000 .. 00000001: Debounce Filter requires 2 adjacent tap detection tests to be the same to trigger a TAP. Response Time >= 0.52ms
      //          00000010: Debounce Filter requires 2 adjacent tap detection tests to be the same to trigger a TAP. Response Time >= 0.78ms
      //          00000011: Debounce Filter requires 2 adjacent tap detection tests to be the same to trigger a TAP. Response Time >= 1.04ms
      //            ...
      //          11111101: Debounce Filter requires 254 adjacent tap detection tests to be the same to trigger a TAP. Response Time >= 66.04 ms
      //          11111110: Debounce Filter requires 255 adjacent tap detection tests to be the same to trigger a TAP. Response Time >= 66.3 ms
      //          11111111: Debounce Filter requires 256 adjacent tap detection tests to be the same to trigger a TAP. Response Time >= 66.56 ms
      
      sendI2cData( MMA7660AccelAddr, MMA7660Accel_PDET,  0x00 ); // 0b
      
      //***********
      //Set "Tap/Pulse Debounce" Register
      // (Exerpted from MMA7660FC Datasheet, Page 21)
      // ---------------------------------------
      //  D7   D6   D5       D4  D3  D2  D1  D0   
      //  ZDA  YDA  XDA  PDTH[4...3...2...1...0]
      // ---------------------------------------
      // XDA (0/1): X-Axis is [Enabled (0) / Disabled (1)] for tap detection
      // YDA (0/1): Y-Axis is [Enabled (0) / Disabled (1)] for tap detection
      // ZDA (0/1): Z-Axis is [Enabled (0) / Disabled (1)] for tap detection
      // PDTH[4:0]: 00000: Tap Detection Threshold is +/- 1 count
      //            00001: Tap Detection Threshold is +/- 2 counts
      //            00010: Tap Detection Threshold is +/- 3 counts
      //             ...
      //            11101: Tap Detection Threshold is +/- 29 counts
      //            11110: Tap Detection Threshold is +/- 30 counts
      //            11111: Tap Detection Threshold is +/- 31 counts
      
      sendI2cData( MMA7660AccelAddr, MMA7660Accel_PD, 0x00 ); // 0b00000000 (Tap Disabled)
      
      //***********
      //Set "Interrupt Setup" Register (what does/doesn't cause an interrupt)
      // (Exerpted from MMA7660FC Datasheet, Page 16)
      // ------------------------------------------------------------
      //   D7     D6       D5      D4      D3      D2      D1    D0   
      // SHINTX  SHINTY  SHINTZ   GINT    ASINT  PDINT   PLINT  FBINT
      // ------------------------------------------------------------
      // SHINTX: (0/1) Shake on the X-axis [does (1) / does not (0)] case an interrupt and [does (1) / does not (0)] set the Shake bit in the TILT register
      // SHINTY: (0/1) Shake on the Y-axis [does (1) / does not (0)] case an interrupt and [does (1) / does not (0)] set the Shake bit in the TILT register
      // SHINTZ: (0/1) Shake on the Z-axis [does (1) / does not (0)] case an interrupt and [does (1) / does not (0)] set the Shake bit in the TILT register
      // GINT:   (0/1) There [is (1) / is not (0)] an automatic interrupt after every measurement (regardless of whether the readings have changed)
      // ASINT:  (0/1) Exiting Auto-Sleep [does (1) / does not (0)] cause an interrupt
      // PDINT:  (0/1) Successful "tap" detection [does (1) / does not (0)] cause an interrupt
      // PLINT:  (0/1) Up/Down/Left/Right (ie Orientation) position change [does (1) / does not (0)] cause an interrupt
      // FBINT:  (0/1) Front/Back (ie Tilt) position change [does (1) / does not (0)] cause an interrupt
      
      sendI2cData( MMA7660AccelAddr, MMA7660Accel_INTSU, 0xFB ); // 0b11111011
      
      //***********
      //Set "Sleep Count" Register (pg. 15)
      // (Exerpted from MMA7660FC Datasheet, Page 15)
      // ------------------------
      //  D7 D6 D5 D4 D3 D2 D1 D0   
      // SC7  6  5  4  3  2  1  0
      // ------------------------
      // SC[7:0]: Sets the maximum count value for the 8-bit internal sleep counter in Auto-Sleep
      //          When Counter > SC[7:0], the IC will switch from Auto-Sleep to Auto-Wake, using
      //            the samples/second set in AWSR[1:0] of the SR register
      //          tl;dr: If it has gone for long enough without having an event, it goes to sleep
      //                 and auto-wakes as often as set by AWSR.
      
      //sendI2cData( MMA7660AccelAddr, MMA7660Accel_SPCNT, 0x01 );   // 0b11111111
  
  //Put Accelerometer into "Active" mode
  sendI2cData( MMA7660AccelAddr, MMA7660Accel_MODE,  0x01 );  // 0b00000001
  
  delay(250);
  
}


//**********************************
// i2cMotorDriver_init: Initializes the I2C Motor Driver
//
void i2cMotorDriver_init(void)
{ //Begins communication with the Motor driver

}


//**********************************
// i2cTouchSensor_init: Initializes the I2C Touch Sensor
//
void i2cTouchSensor_init(void)
{ //Begins communication with the Touch Sensors

  // From http://www.prizepony.us/bin/arduino_mpr121.zip by Rory Nugent
  
  // Section A
  // This group controls filtering when data is > baseline.
  sendI2cData( i2cTouchSensorAddr, 0x2B, 0x01 );
  sendI2cData( i2cTouchSensorAddr, 0x2C, 0x01 );
  sendI2cData( i2cTouchSensorAddr, 0x2D, 0x00 );
  sendI2cData( i2cTouchSensorAddr, 0x2E, 0x00 );

  // Section B
  // This group controls filtering when data is < baseline.
  sendI2cData( i2cTouchSensorAddr, 0x2F, 0x01 );
  sendI2cData( i2cTouchSensorAddr, 0x30, 0x01 );
  sendI2cData( i2cTouchSensorAddr, 0x31, 0xFF );
  sendI2cData( i2cTouchSensorAddr, 0x32, 0x02 );

  // Section C
  // This group sets touch and release thresholds for each electrode
  sendI2cData( i2cTouchSensorAddr, 0x41, i2cTouchSensor_TOUCH_THRESH );
  sendI2cData( i2cTouchSensorAddr, 0x42, i2cTouchSensor_REL_THRESH   );
  sendI2cData( i2cTouchSensorAddr, 0x43, i2cTouchSensor_TOUCH_THRESH );
  sendI2cData( i2cTouchSensorAddr, 0x44, i2cTouchSensor_REL_THRESH   );
  sendI2cData( i2cTouchSensorAddr, 0x45, i2cTouchSensor_TOUCH_THRESH );
  sendI2cData( i2cTouchSensorAddr, 0x46, i2cTouchSensor_REL_THRESH   );
  sendI2cData( i2cTouchSensorAddr, 0x47, i2cTouchSensor_TOUCH_THRESH );
  sendI2cData( i2cTouchSensorAddr, 0x48, i2cTouchSensor_REL_THRESH   );
  sendI2cData( i2cTouchSensorAddr, 0x49, i2cTouchSensor_TOUCH_THRESH );
  sendI2cData( i2cTouchSensorAddr, 0x4A, i2cTouchSensor_REL_THRESH   );
  sendI2cData( i2cTouchSensorAddr, 0x4B, i2cTouchSensor_TOUCH_THRESH );
  sendI2cData( i2cTouchSensorAddr, 0x4C, i2cTouchSensor_REL_THRESH   );

  // Section D
  // Set the Filter Configuration
  // Set ESI2
  sendI2cData(i2cTouchSensorAddr, 0x5D, 0x04);

  // Section E
  // Electrode Configuration
  // Enable 6 Electrodes and set to run mode
  // Set ELE_CFG to 0x00 to return to standby mode
  sendI2cData(i2cTouchSensorAddr, 0x5E, 0x06);		// Enable first 6 electrodes

  // Section F
  // Enable Auto Config and auto Reconfig
  /*sendI2cData(ATO_CFG0, 0x0B);
   	sendI2cData(ATO_CFGU, 0xC9);	// USL = (Vdd-0.7)/vdd*256 = 0xC9 @3.3V
   	sendI2cData(ATO_CFGL, 0x82);	// LSL = 0.65*USL = 0x82 @3.3V
   	sendI2cData(ATO_CFGT, 0xB5);*/  // Target = 0.9*USL = 0xB5 @3.3V

  delay(250);

}


//**********************************
// rgbLed_init: Initializes the Chainable RGB LEDs
//
void rgbLed_init(void)
{ //Begins communication with the RGB LEDs
  Send32Zero();
  DataDealWithAndSend(0, 0, 0);
  DataDealWithAndSend(0, 0, 0);
  //Add more here if you add more RGB LEDs
  Send32Zero();        
}


//**********************************
// i2cOledDisplay_init: Initializes the OLED Display
//
void i2cOledDisplay_init(void)
{ //Begins communication with the Display
  delay(50);
  sendI2cData( oledAddr, oled_COMMAND, 0xae ); //display off
  delay(50); 
  //  sendcommand(0xa0);		//seg re-map 0->127(default)
  //  sendcommand(0xa1);		//seg re-map 127->0
  //  sendcommand(0xc8);
  //        delay(1000);
  sendI2cData( oledAddr, oled_COMMAND, 0xaf ); //display on
  delay(50); 
  oled_clearDisplay();

  sendI2cData( oledAddr, oled_COMMAND, 0x20 );   //Set Memory Addressing Mode
  sendI2cData( oledAddr, oled_COMMAND, 0x02 );   //Set Memory Addressing Mode ab Page addressing mode(RESET)  
  sendI2cData( oledAddr, oled_COMMAND, 0xa6 );   //Set Normal Display (default)

  //Show Title Screen
  setXY(1,2);
  sendStr(fill_string1);
  setXY(2,5);
  sendStr(fill_string2);
  setXY(4,3);
  sendStr(fill_string3);
  setXY(5,3);
  sendStr(fill_string4);
  delay(1000);
  //    setXY(7,3);
  //   sendStr(fill_string5);
  //  sendChar(0x2e);
}


//**********************************
// OLED Display Functions
//**********************************

//**********************************
// setXY: Set the current position on the OLED display to (row, col)
// -> unsigned char row     - sets the current OLED Row    (0-7)
// -> unsigned char col     - sets the current OLED Column (0-17)
//
void setXY(unsigned char row, unsigned char col) {
  sendI2cData( oledAddr, oled_COMMAND, 0xb0+row );               //set page address
  sendI2cData( oledAddr, oled_COMMAND, 0x00+(8*col&0x0f) );      //set low col address
  sendI2cData( oledAddr, oled_COMMAND, 0x10+((8*col>>4)&0x0f) ); //set high col address
}


//**********************************
// sendChar: Display a character on the OLED display at the current position
// -> unsigned char ascii     - (Default: 0): Character to be displayed on the OLED display
//
void sendChar(unsigned char ascii=0) {
  unsigned char i=0; 
  for(i=0;i<8;i++) {
    sendI2cData( oledAddr, oled_DATA, myFont[ascii-0x20][i] );
  }
}


//**********************************
// sendStr: Display text stored in 'char* string' on OLED display at the current position
// -> char* string     - Text to be displayed on the OLED display
//
void sendStr(char* string) {
  unsigned char i=0;
  //setXY(0,0);      
  while(*string) {
    for(i=0;i<8;i++) {
      sendI2cData( oledAddr, oled_DATA, myFont[*string-0x20][i] );
//      delay(2);
    }
    *string++;
  }
}


//**********************************
// oled_clearDisplay: Clear the OLED display
//
void oled_clearDisplay(void) {
  unsigned char i,k;
  for(k=0;k<8;k++) {	
    setXY(k,0);    
    {
      for(i=0;i<128;i++) { //clear all COL
        sendI2cData( oledAddr, oled_DATA, 0 );
//        delay(2);
      }
    }
  }
  delay(50);
}


//**********************************
// SendI2cData (Address, Mode, Data)
//   A generic I2C data sender
//**********************************

//**********************************
// sendI2cData: Send I2C data to a slave device
// -> unsigned char i2cAddress   - What I2C Address should we send data to?
// -> unsigned char commandMode  - What command mode (register) should be we be using?
// -> unsigned char myData       - Byte 1 to be sent to [commandMode]
// -> unsigned char myData2      - (OPTIONAL) Byte 2 to be sent to [commandMode]
// -> unsigned char myData3      - (OPTIONAL) Byte 3 to be sent to [commandMode]

//Overload sendI2cData for one myData byte
void sendI2cData(unsigned char i2cAddress, unsigned char commandMode, byte myData) {
/*  Wire.beginTransmission(i2cAddress);
  Wire.send(commandMode);
  Wire.send(myData);
  Wire.endTransmission();*/
  sendI2cData(i2cAddress, commandMode, myData, NULL, NULL);
}

//Overload sendI2cData for two myData bytes
void sendI2cData(unsigned char i2cAddress, unsigned char commandMode, byte myData, byte myData2) {
/*  Wire.beginTransmission(i2cAddress);
  Wire.send(commandMode);
  Wire.send(myData);
  Wire.send(myData2);
  Wire.endTransmission();*/
  sendI2cData(i2cAddress, commandMode, myData, myData2, NULL);
}

//The previous two versions of "sendI2cData" are there so that the number of "myData" bytes sent can be optional
//TODO: Maybe in future accept a byte array, and loop through them? Not enough time at the moment to do that.
void sendI2cData(unsigned char i2cAddress, unsigned char commandMode, byte myData, byte myData2, byte myData3) {
  Wire.beginTransmission(i2cAddress);
  Wire.send(commandMode);
  Wire.send(myData);
//  Wire.send(myData2);
//  Wire.send(myData3);
  if (myData2) Wire.send(myData2);
  if (myData3) Wire.send(myData3);
  Wire.endTransmission();
}


//**********************************
// requestI2cData: Request [numBytes] bytes from [i2cAddress]'s [commandMode] register
// -> unsigned char i2cAddress   - What I2C Address should we request data from?
// -> unsigned char commandMode  - What register should we request data from?
// -> unsigned char numBytes     - How many bytes should we request?
//
// returns: char   - A byte containing the response from the I2C Slave Device
//  //TODO: Does numBytes == 2 return 1 or 2 bytes?
char requestI2cData(unsigned char i2cAddress, unsigned char commandMode, byte numBytes) {
  char incoming;
  
  Wire.beginTransmission(i2cAddress);
  Wire.send(commandMode);
  Wire.requestFrom(i2cAddress,numBytes);

  if (1 <= Wire.available()) {
    incoming = Wire.receive();
  }
  
  Wire.endTransmission();
  
  return incoming;
}


//**********************************
// I2C Motor Driver Functions
//**********************************

//**********************************
// setMotors: Set one motor's direction and speed
// -> unsigned char motorName       - Which motor? (MOTOR_1 or MOTOR_2)
// -> unsigned char motorDirection  - Which direction should the motor be turning? 
//                                        MOTOR_FWD == Forward
//                                        MOTOR_REV == Reverse (Backward)
//                                        MOTOR_OFF == Off/None/Disable
// -> unsigned char motorSpeed      - What speed should the motor be running at? (0-255)
//
void setMotor(unsigned char motorName, unsigned char motorDirection, unsigned char motorSpeed) {
  sendI2cData(i2cMotorDriverAddr, motorName, motorDirection, motorSpeed );
}


//**********************************
// setMotors: Set both motors' speeds
// -> unsigned char motorASpeed   - The speed (0-255) that MOTOR_1 should be running at
// -> unsigned char motorBSpeed   - The speed (0-255) that MOTOR_2 should be running at
//
void setMotors(unsigned char motorASpeed, unsigned char motorBSpeed) {
  sendI2cData(i2cMotorDriverAddr, i2cMotorDriver_SETPWM, motorASpeed, motorBSpeed );
}


//**********************************
//RGB LED Functions
//**********************************

//**********************************
// ClkProduce: Produce a clock on the clockPin
// 
void ClkProduce(void)
{
  digitalWrite(clockPin, LOW);
  delayMicroseconds(20); 
  digitalWrite(clockPin, HIGH);
  delayMicroseconds(20);   
}


//**********************************
// Send32Zero: Send 32 "LOW" values (0) to the RGB LED modules.
//             This is used to initialize and update the modules
//               and their values
// 
void Send32Zero(void)
{
  unsigned char i;

  for (i=0; i<32; i++) {
    digitalWrite(dataPin, LOW);
    ClkProduce();
  }
}


//**********************************
// TakeAntiCode: Calculate the Anti-code for the RGB LED message checksum
// -> unsigned char dat     - data to be checksummed
//
// returns: unsigned char   - checksum of the data
//
unsigned char TakeAntiCode(unsigned char dat)
{
  unsigned char tmp = 0;

  if ((dat & 0x80) == 0) {
    tmp |= 0x02; 
  }

  if ((dat & 0x40) == 0) {
    tmp |= 0x01; 
  }

  return tmp;
}


//**********************************
// DatSend: Clock out the data in "dx" to the RGB LEDs
// -> unsigned long int dx   - data to be clocked out (32 bits)
//
// gray data
void DatSend(unsigned long int dx)
{
  unsigned char i;

  for (i=0; i<32; i++) {
    if ((dx & 0x80000000) != 0) {
      digitalWrite(dataPin, HIGH);
    } 
    else {
      digitalWrite(dataPin, LOW);
    }

    dx <<= 1;
    ClkProduce();
  }
}


//**********************************
// DataDealWithAndSend: Process data for the chainable RGB LEDs, and send it to them.
// -> unsigned char r    - the RED value   (0-255)
// -> unsigned char g    - the GREEN value (0-255)
// -> unsigned char b    - the BLUE value  (0-255)
void DataDealWithAndSend(unsigned char r, unsigned char g, unsigned char b)
{
  unsigned long int dx = 0;

  dx |= (unsigned long int)0x03 << 30;             // highest two bits 1，flag bits
  dx |= (unsigned long int)TakeAntiCode(b) << 28;
  dx |= (unsigned long int)TakeAntiCode(g) << 26;	
  dx |= (unsigned long int)TakeAntiCode(r) << 24;

  dx |= (unsigned long int)b << 16;
  dx |= (unsigned long int)g << 8;
  dx |= r;

  DatSend(dx);
}


//**********************************
// rgbPoliceLights: Flash the RGB LED(s) in a police-car-like pattern
// 
void rgbPoliceLights(void) {
  static int rgbPattern;
  
  //Change the "1" in here to your highest case from the switch() below
  if (rgbPattern > 1) rgbPattern = 0;
  
  switch(rgbPattern) {
    case 0:
      Send32Zero(); // begin
      DataDealWithAndSend(255, 0, 0); // first node data
      DataDealWithAndSend(0, 0, 255); // second node data
      Send32Zero();  // send to update data   
      break;
    case 1:
      Send32Zero(); // begin
      DataDealWithAndSend(0, 0, 255); // first node data
      DataDealWithAndSend(255, 0, 0); // second node data
      Send32Zero();  // send to update data   
      break;
      //You can add different pattens or steps in the pattern here:
/*    case 2:
      Send32Zero(); // begin
      DataDealWithAndSend(255, 0, 0); // first node data
      DataDealWithAndSend(0, 0, 255); // second node data
      Send32Zero();  // send to update data    
      break;
    case 3:
      Send32Zero(); // begin
      DataDealWithAndSend(0, 0, 255); // first node data
      DataDealWithAndSend(255, 0, 0); // second node data
      Send32Zero();  // send to update data    
      break;
    case 4:
      Send32Zero(); // begin
      DataDealWithAndSend(255, 0, 0); // first node data
      DataDealWithAndSend(0, 0, 255); // second node data
      Send32Zero();  // send to update data    
      break;
    case 5:
      Send32Zero(); // begin
      DataDealWithAndSend(0, 0, 255); // first node data
      DataDealWithAndSend(255, 0, 0); // second node data
      Send32Zero();  // send to update data    
      break;
    case 6:
      Send32Zero(); // begin
      DataDealWithAndSend(255, 0, 0); // first node data
      DataDealWithAndSend(0, 0, 255); // second node data
      Send32Zero();  // send to update data       
      break;
    case 7:
      Send32Zero(); // begin
      DataDealWithAndSend(0, 0, 255); // first node data
      DataDealWithAndSend(255, 0, 0); // second node data
      Send32Zero();  // send to update data    
      break;*/
    default:
      //Just in case something happens and we end up over our target number of cases...
      rgbPattern = -1;
  }
  
  rgbPattern++;
}


//***************
// setup: Set up the pins, run initializations, etc
//
void setup()
{
  Serial.begin(9600);  // start serial for output
  Serial.println("Make ready my ship, General!");
  
//  delay(50);
  
  Wire.begin(); //Start I2C
  
  //** Set up the OLED Display
  i2cOledDisplay_init();
  
  Serial.println("OLED Active");

  //Init other devices
  MMA7660Accel_init();
  i2cMotorDriver_init();
  i2cTouchSensor_init();

  //** Set up pins
  
  //Chainable RGB LED Pins
  pinMode(dataPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  rgbLed_init();

  //Vibrator Pin
  pinMode(vibratingMotor, OUTPUT);

  //Motion Sensor Pin
  pinMode(motionSensor, INPUT);
  
  //Sound Sensor Pin
  pinMode(soundSensor, INPUT);     //A0
  pinMode(proximitySensor, INPUT); //A2
  
  //Initialize Sound Recorder pins and set sound to OFF
  pinMode(soundRecorderPin1, OUTPUT);
//  digitalWrite(soundRecorderPin1, LOW);
  pinMode(soundRecorderPin2, OUTPUT);
//  digitalWrite(soundRecorderPin2, LOW);
  
  //I2C Touch Sensor IRQ Input
  pinMode(i2cTouchSensor_IRQ, INPUT);
  digitalWrite(i2cTouchSensor_IRQ, HIGH);
  pinMode(MMA7660Accel_IRQ,   INPUT);
  digitalWrite(MMA7660Accel_IRQ, HIGH);
  
//  delay(50);
   
  //Attach Accelerometer and Button-Press interrupts
  attachInterrupt(0, onButtonPress, FALLING); 
  attachInterrupt(1, onAcceleration, FALLING);
  Serial.println("Interrupts attached.");

  Serial.println("It's time to show those dogs what we are made of!");
  
    //Show Main Menu
  showMainMenu();  
  
//  menuSelect(1);
//createGraph("Foo", 100, -100);
}


//**********************************
// readAccel: Read the acceleration from the accelerometer and save it to
//            the AccelX, AccelY, and AccelZ variables
void readAccel(void)
{
  char val[3];
  char ret[3];
  int count = 0;

//  accelChanged = 0;
  
  val[0] = val[1] = val[2] = 64;
  Wire.requestFrom(0x4c, 3); // request 3 bytes from Accelerometer

  while(Wire.available())  
  {
    if(count < 3)
      while ( val[count] > 63 )  // reload it, it is bad
      {
        val[count] = Wire.receive();

      }
    count++;
  }
  
  AccelX = ((char)(val[0]<<2))/4;
  AccelY = ((char)(val[1]<<2))/4;
  AccelZ = ((char)(val[2]<<2))/4;
}


//**********************************
// vibratingMotor_buzz: Buzz the vibrating motor for "length" milliseconds
// -> int    length   - the length of time (in milliseconds) we want the vibrator to buzz
void vibratingMotor_buzz(int length)
{ 
//  Serial.println("Buzzing Vibrator");
  digitalWrite(vibratingMotor, 1);
  delay(length);
  digitalWrite(vibratingMotor, 0);
  tone(buzzerPin, NOTE_C1, length);  // NOTE_C4 == 262, however I've included pitches.h to let you make
                                       //                 your own melodies in case you aren't using the
                                       //                 Sound Recorder Twig. 
                                       //                 See http://arduino.cc/en/Tutorial/Tone for more
                                       //                 details.
}


//***************************
// soundRecorder_playSound: Plays the sound corresponding to the combination of pin 1 and pin 2
// -> char    pin1   - the value we want to set on pin 1 (0 or 1)
// -> char    pin2   - the value we want to set on pin 2 (0 or 1)
void soundRecorder_playSound(int pin1, int pin2)
{ //Where "sound" is one of (0,0); (0,1); (1,0); (1,1)
  // playSound (0,0):  Turns sound OFF
  // playSound (0,1):  Plays prerecorded sound #2
  // playSound (1,0):  Plays prerecorded sound #3
  // playSound (1,1):  Plays prerecorded sound #4

  digitalWrite(soundRecorderPin1, pin1);
  digitalWrite(soundRecorderPin2, pin2);
}


//***************************
// showMainMenu: Displays the main menu.
//
void showMainMenu(void) {
  delay(50);
//  Serial.println("Showing Main Menu");

  //Put menu items in this array. They MUST have \0 after each one
  char* temp[] = { "Auto Mode\0", 
                   "Remote Ctrl\0", 
                   "Cat Spooker\0", 
                   "Cat Taunter\0", 
                   "Motor Test\0", 
                   "Sensor Data\0"
                 };
  numScreenRows = sizeof(temp)/sizeof(int);
//  numScreenRows = 5; //0..5 == 6 menu items
//  Serial.println("Creating Screen...");
  cursorActive = 1;
  
  delay(50);
  createScreen(temp, 0, "Main Menu\0", 0x2d, 3, 2, 1, 0);  //Show a screen with the temp[] rows, a screenId of 0, the title "Main Menu", the rows starting at X-axis position 3, with a cursor, defaulting to the first option
  delay(50);
  
//  Serial.println("Done");
}


//***************************
// setArrow: Puts the 'current selection' menu arrow on a certain row
// -> int    row   - the row we want the cursor to appear on
void setArrow(int row) {
  unsigned char i;

  cursorPosition = row;  //Store the row we're currently on
  cursorActive = 1;      //We just set a cursor, so we want it to show

  //Remove arrow from each of the possible spots
  for(i=0;i<8;i++) {
//    Serial.print(i);
//    delay(50);
    setXY(i,0);
    
/*    if (i == row) { 
      sendStr("=>\0");
    } else {
      sendStr("  \0");
    }*/
    if (i == row) {       
      sendChar(0x3D);
      setXY(i,1);
      sendChar(0x3E);
    } else {
      sendChar(0x20);
      setXY(i,1);
      sendChar(0x20);
    }

  }
}


//***************************
// plotValue: Plots a value on a screen that has been created by createGraph
// -> int    x   - the X value (Where on the graph do we want the value to show?)
// -> int    y   - The Y value (What value are we showing?)
void plotValue(int x, int y) {
  //TODO
}


//***************************
// createGraph: Clears the screen and makes a graph
// -> char*  title    - The title of the graph (currently unused)
// -> char   maxVal   - The maximum value of the graph 
// -> int    minVal   - The minimum value of the graph
//
//  Currently there are 3 types of graphs:
//    1) minVal == 0, where the Y-axis goes from 0 to maxVal
//    2) minVal > 0, where the Y-axis goes from minVal to maxVal, with no positive/negative delimiter at y = 0
//    3) minVal < 0, where the Y-axis crosses y = 0, so there is a line to differentiate between positive and negative
//  The bitmaps createGraph creates are located in the project directory if you wish to modify them.
//  The bit data is stored in data.c and read out to the screen.
void createGraph(char* title, int maxVal, int minVal) {
  
//    sendI2cData( oledAddr, oled_COMMAND, 0xae ); //display off
//  delay(50); 
  //  sendcommand(0xa0);		//seg re-map 0->127(default)
  //  sendcommand(0xa1);		//seg re-map 127->0
  //  sendcommand(0xc8);
  //        delay(1000);
//  sendI2cData( oledAddr, oled_COMMAND, 0xaf ); //display on
//  delay(50); 
/*  oled_clearDisplay();
  
  sendI2cData(oledAddr, oled_COMMAND, 0xa6);		//normal display
  
  delay(50);
  
//  Serial.println("Here goes a graph!");
  
  sendI2cData(oledAddr, oled_COMMAND, 0xae);		//display off
  sendI2cData(oledAddr, oled_COMMAND, 0x20);            //Set Memory Addressing Mode
  sendI2cData(oledAddr, oled_COMMAND, 0x01);            //Set Memory Addressing Mode: Vertical Addressing

  delay(50);
    //Put layout in this array.
//  if (minVal < 0) { //Let's do a +/- graph
      for(int i=0;i<128*8;i++) { // write a 128*64 picture
        sendI2cData(oledAddr, oled_DATA, pos_neg_graph[i]);
      }
/*  } else if (minVal > 0) { //Non-baseline Positive graph
      for(int i=0;i<128*8;i++) { // write a 128*64 picture
        sendI2cData(oledAddr, oled_DATA, max_min_graph[i]);
      }
  } else {//Regular positive graph, base 0
    for(int i=0;i<128*8;i++) { // write a 128*64 picture
      sendI2cData(oledAddr, oled_DATA, positive_graph[i]);
      delay(1);
    }
  }*/
  
//  sendI2cData(oledAddr, oled_COMMAND, 0xaf);

//  createScreen(temp, 99, title, 0x20, 0, 0, 0, 0); //"Space"/blank underline
}


//***************************
// createScreen: Makes a screen; either a menu or a regular screen of text
// -> char** screenLines    - An array of the row texts (MAX 6; ie. 0-5)
// -> char   screenId       - The ID number of the screen we're showing 
//                            (So we can keep track of what to do when buttons are pressed)
// -> char*  screenTitle    - The title of the menu
// -> char   menuUnderlineChar - The character we use to underline the title (typically 0x2d, "-")
// -> char   xOffset        - What position does text start in on the X-Axis? This lets you leave room for the cursor 
//                              (either "0" if you're not using a cursor, or "2" if you're using a 2-character cursor)
// -> char   yOffset        - What position does text start in on the Y-Axis? This leaves room for the title.
// -> char   showCursor     - Do we show the cursor? (ie, is this a menu row?)
// -> int    default_row    - The default menu item to point at (0-5 with a title, 0-7 without)
void createScreen(char** screenLines, char screenId, char* screenTitle, char menuUnderlineChar, char xOffset, char yOffset, char showCursor, int defaultRow ) {
  shutDownEVERYTHING = 0; //Reset the shutdown flag
  cursorActive = 0;       //Disable the cursor for now. It will be activated later by setArrow if showCursor == TRUE
  currScreen = screenId;  //Store the ID of the screen or menu we're currently showing
  
//  delay(250);
  
  //Clear the display and get it ready for our new screen
  oled_clearDisplay();
//  delay(250);
  
  sendI2cData(oledAddr, oled_COMMAND, 0x20);   //Set Memory Addressing Mode
  sendI2cData(oledAddr, oled_COMMAND, 0x02);   //Set Memory Addressing Mode ab Page addressing mode(RESET)  
  sendI2cData(oledAddr, oled_COMMAND, 0xa6);   //Set Normal Display (default)

//  delay(250);

  //If we have a title, display it
  if (screenTitle) { 
    //Let's figure out how to 'center' the title
    //  17 characters across, so (17 - strlen(title) - 1) / 2
    char startPos = (17 - strlen(screenTitle)) / 2;
    //If you don't want the title auto-centered, just replace the above with char startPos = 3; (or your desired start location)
    
    //Send the menu title  
    setXY(0,startPos);
    sendStr(screenTitle);
  
    //Send a title underline that's the same length as the title
    setXY(1,startPos);
    for (int i=0; i < strlen(screenTitle); i++) {
      sendChar(menuUnderlineChar); //Send the underline character we want 
    }
    
//    if (numScreenRows <= 5) //If we want to 
//    titleHeight = 7 - numScreenRows; //Our title is 2 rows tall (alternate: start displaying regular text rows on line 2.)
    titleHeight = yOffset; //Our title is 2 rows tall (alternate: start displaying regular text rows on line 2.)

  } else {
    titleHeight = 0;
  }

  //Display each row of text we have, starting on the next available line
  for (int i = 0; i < numScreenRows; i++) {
    //Show the screenLines[0], [1], etc starting on the next available line.. line 2 if we have a title, line 0 otherwise."
    //  and display it 3 characters in to leave room for the "=> " cursor if this is a menu
    setXY(i+yOffset, xOffset);
//    setXY(i+titleHeight, xOffset);
    sendStr(screenLines[i]);
  }  

  //TODO: Show Title *after* the lines data so we can write over stuff?
//  delay(250);

  if (showCursor == 1) setArrow(defaultRow + yOffset); //Point an arrow at the proper spot if needed
  
//  delay(250);
}

//Overload the real createScreen with a shorter version for non-menu but titled screens with a custom underline character
void createScreen(char** screenLines, char screenId, char* screenTitle, char menuUnderlineChar) {
  createScreen(screenLines, screenId, screenTitle, menuUnderlineChar, 0, 2, 0, 0);
}

//Overload the real createScreen with a shorter version for non-menu but titled screens (with default 0x2d ("-") underline)
void createScreen(char** screenLines, char screenId, char* screenTitle) {
  createScreen(screenLines, screenId, screenTitle, 0x2d, 0, 2, 0, 0);
}

//Overload the real createScreen with a shorter version for non-menu and non-titled screens
void createScreen(char** screenLines, char screenId) {
  createScreen(screenLines, screenId, NULL, NULL, 0, 0, 0, 0);
}


//***************************
// buttonHandler: Handles button presses from the I2C Touch Sensors
// -> byte what    - Which button was pressed? (0 to 3)
//                     0 == Up                == BUTTON_UP
//                     1 == Down              == BUTTON_DOWN
//                     2 == OK     (Proceed)  == BUTTON_OK
//                     3 == Cancel (Go Back)  == BUTTON_CANCEL
//
void buttonHandler(byte what) {

  int cursorStart = titleHeight;                //What is the top line the cursor can go to?    (The menu items start 2 down because of the title and underlining)
  int cursorEnd = numScreenRows + titleHeight - 1;   //What is the bottom line the cursor can go to? (The menu items are offset by 2 because of the title and underlining)

  switch (what) {
    case BUTTON_UP:
//      Serial.println("UP Button pushed.");
      //If the cursor is turned on and we can move the cursor up one
      if (cursorActive && cursorPosition > cursorStart) {
        setArrow(cursorPosition - 1);
      }
      break;
    case BUTTON_DOWN:
//      Serial.println("DOWN Button pushed.");
      //If the cursor is turned on and we can move the cursor down one
      if (cursorActive && cursorPosition < cursorEnd) {
        setArrow(cursorPosition + 1);
      }
      break;
    case BUTTON_OK:
//      Serial.println("OK Button pushed.");
      //Start whatever we're pointing at.
//      if (currScreen == 1) { //If we're watching the sensors list, OK == EXIT
//        shutDownEVERYTHING = 1; //This is enough to shut down the sensors list
//      } else {
        cursorActive = 0;
        menuSelect(cursorPosition - titleHeight); //Offset by the titleHeight so we get a number of 0-n for which menu item we're clicking
//      }
      break;
    case BUTTON_CANCEL:
//      Serial.println("Cancel Button pushed.");
      //Do some kind of "Stop everything". Reset?
      shutDownEVERYTHING = 1; //This is enough to shut down the Sensors List (Screen 1)
                              //            "               the ... (Screen 2)
      break;
    default:
      Serial.println("buttonHandler doesn't know what to do!");    
  }
}


//***************************
// motorTest: Performs a test of a motor and displays the current action on the OLED screen
// -> unsigned char motorNum        - Which motor? (MOTOR_1 or MOTOR_2)
// -> unsigned char motorDirection  - Which direction should the motor be turning? 
//                                        MOTOR_FWD == Forward
//                                        MOTOR_REV == Reverse (Backward)
//                                        MOTOR_OFF == Off/None/Disable
// -> unsigned char motorSpeed      - What speed should the motor be running at? (0-255)
//
void motorTest(unsigned char motorNum, unsigned char motorDirection, unsigned char motorSpeed) {
  char temp[17];
  char motorNumber;
  
  if (motorNum == MOTOR_1) {
    motorNumber = 1;
    setXY(3,0);
  } else if (motorNum == MOTOR_2) {
    motorNumber = 2;
    setXY(4,0);
  }

  //Big long ugly nested inline if here.
  sprintf(temp, "Motor %d: %s %3d", motorNumber, 
             (motorDirection == MOTOR_FWD ? "FWD" : (motorDirection == MOTOR_REV ? "REV" : "OFF")),
             motorSpeed);
  sendStr(temp);
    delay(50);
  sendI2cData(i2cMotorDriverAddr, motorNum, motorDirection, motorSpeed);
//  delay(50); //Some Arduino builds need this. Weird, but okay.
    
}


//***************************
// menuSelect: Perform the action associated with menu item #[menuItem]
// -> char menuItem        - Which menu item are we executing?
//
//  TODO: Replace this with a function pointer array that's tied to the Menu Item Number? (Not likely)
//
void menuSelect(char menuItem) {

//  Serial.println(menuItem);
  shutDownEVERYTHING = 0;
  
  switch(menuItem) {
    case 0: //"Auto Mode": Autonomous Mode
      doAutonomous();
      break;
    case 1: //"Remote Control": Manual control by Bluetooth or USB
      doRemoteControl();
      break;
    case 2: //"Cat Spooker": Sit still and wait until something trips the PIR sensor, then fire up the engine and go for a spin
      doCatSpooker();
      break;
    case 3: //
      doCatTaunter();
      break;
    case 4: //"Motor Test": Test the motors in various forward/reverse configurations
      doMotorTest();
      break;   
    case 5: //"Sensor Data"
      doSensorView();
      break;
//    case 6: //Not implemented because it wouldn't be shown with titleHeight = 2
//    case 7: //Not implemented because it wouldn't be shown with titleHeight = 2
    default:
      Serial.println("MenuSelect doesn't know what to do!");
  }
 
//  delay(250); 

  //***Clean up after ourselves
  Timer1.detachInterrupt(); //Stop the RGB LEDs
      
  //Turn off the RGB LEDs
  Send32Zero(); // begin
  DataDealWithAndSend(0, 0, 0); // first node data
  DataDealWithAndSend(0, 0, 0); // second node data
  Send32Zero();  // send to update data    
      
  soundRecorder_playSound(0,0); //Stop the Sounds

  //Shut down the motors (as a failsafe)
  sendI2cData(i2cMotorDriverAddr, MOTOR_1, MOTOR_OFF, 255);
  sendI2cData(i2cMotorDriverAddr, MOTOR_2, MOTOR_OFF, 255);

  //Show the main menu
  showMainMenu();
}

//***************************
// doAutonomous: Activate Autonomous Mode
//
void doAutonomous(void) {
      int motionSensorValue = 0;
      int proximitySensorValue = 0;  
      char* lines[] = {       "\0",     "\0",      "\0",      "\0",      "\0",      "'Cancel' to Exit\0"    };
      numScreenRows = sizeof(lines)/sizeof(int);
      createScreen(lines, 3, "Autonomous Mode");  //Create a basic screen with the lines from lines[], screenId 3 (Cat Taunter)  

      soundRecorder_playSound(1,0); //Start up the engine!
      delay(3500);  //My sound clip is approx 3.5 seconds long
      soundRecorder_playSound(0,0); //Stop the clip
      delay(100);   //Give it time to detect the "OFF" command before issuing a new one
      
      if (touchChanged == 1) handleTouch(); //Check for touches (ie Cancel)

      while (shutDownEVERYTHING == 0) {
//        rgbPoliceLights(); //Flash the RGB LEDs
        soundRecorder_playSound(0,1); //Make sure the "Idle" sound is playing. It doesn't matter if it is already, it'll just continue

        //Turn on the police lights
        Timer1.initialize(250000); //Timer set at 0.25s
        Timer1.attachInterrupt(rgbPoliceLights);

        //All ahead, full!
        setMotor(MOTOR_1, MOTOR_OFF, 0);   //Turn off steer motor just in case it's on
        setMotor(MOTOR_2, MOTOR_FWD, 255); //All ahead!
        
        proximitySensorValue = analogRead(proximitySensor);

        while (proximitySensorValue < proximitySensorThreshold && shutDownEVERYTHING == 0) { //Keep going until we see something in our way
          proximitySensorValue = analogRead(proximitySensor); //Check for obstacles
          if (touchChanged == 1) handleTouch();          //Check for touches
        }
        
        if (shutDownEVERYTHING == 1) break;
        
//        Serial.print("Motion Detected.");
        
        setMotor(MOTOR_2, MOTOR_OFF, 0); //Stop

//        Serial.println("HONK! HONK!");   

        soundRecorder_playSound(0,0); //Stop the sound
        delay(100);   //Give it time to detect the "OFF" command before issuing a new one        
        soundRecorder_playSound(1,1); //HONK! HONK!
        delay(500);
        soundRecorder_playSound(0,0); //Stop the sound
        delay(100);   //Give it time to detect the "OFF" command before issuing a new one  
               
//        Serial.println("Backing Up...");
        setMotor(MOTOR_1, MOTOR_FWD, 255); //Steer to the right
        setMotor(MOTOR_2, MOTOR_REV, 128); //Back Up
        
        //Back up for 1 second
        for (int i = 0; i < 10; i++) {
//          rgbPoliceLights(); //Keep the RGB LEDs going
          delay(100);        //Wait a bit
        }
        
//        Serial.println("Straightening Up...");
        setMotor(MOTOR_1, MOTOR_OFF, 0); //Steer straight (The steer wheels self-correct with a spring)
        setMotor(MOTOR_2, MOTOR_OFF, 0); //Stop
        
        delay(250); //Hold on for a second.. Because we can.
        
//        Serial.println("Aaaaand... Go!");
        setMotor(MOTOR_2, MOTOR_FWD, 255); //Go!

        //Now we loop again to go.
        
        if (touchChanged == 1) handleTouch();
      }     
}

//***************************
// doCatSpooker: Activate the "Cat Spooker"
//
void doCatSpooker(void) {
      int motionSensorValue;
      
      char* lines[] = {       "\0",     "Waiting for\0",      "        Kitty...\0",      "\0",      "\0",      "'Cancel' to Exit\0"    };
      numScreenRows = sizeof(lines)/sizeof(int);
      createScreen(lines, 5, "Cat Spooker");  //Create a basic screen with the lines from lines[], screenId 3 (Cat Taunter)  
      
      //Wait for a PIR signal
      while (shutDownEVERYTHING == 0 && motionSensorValue == 0) { //"Cancel" button will set shutDownEVERYTHING == 1
        if (touchChanged == 1) handleTouch();  //Handle the touch event
        //No accelerometer checking since we're not worried about it in this mode. Kitty's going to sneak up on us.
  
        motionSensorValue = digitalRead(motionSensor); //Get a new value
      }
      
      if (shutDownEVERYTHING == 1) return;
      
      //Make some noise!
      soundRecorder_playSound(1,0); //Start up the engine!
      delay(3500);  //My sound clip is approx 3.5 seconds long
      soundRecorder_playSound(0,0); //Stop the clip
      delay(100);   //Give it time to detect the "OFF" command before issuing a new one
      
      //Run forward a bit
      setMotor(MOTOR_2, MOTOR_FWD, 255);
      delay(1000);
      setMotor(MOTOR_2, MOTOR_OFF, 0);
}

//***************************
// doCatTaunter: Activate the "Cat Taunter"
//
void doCatTaunter(void) {
      int motionSensorValue;
  
      char* lines[] = {       "\0",      "\0",      "\0",      "\0",      "\0",      "'Cancel' to Exit\0"    };
      numScreenRows = sizeof(lines)/sizeof(int);
      createScreen(lines, 6, "Cat Taunter");  //Create a basic screen with the lines from lines[], screenId 3 (Cat Taunter)  
  
      delay(5000); //Wait 5 seconds before starting so we can get away :)
  
      while (shutDownEVERYTHING == 0) { //"Cancel" button will set shutDownEVERYTHING == 1
        if (touchChanged == 1) handleTouch();  //Handle the touch event
        
        motionSensorValue = digitalRead(motionSensor); //Get a new value
        
        if (motionSensorValue == 0) { //No one's here... Let's make some noise!
          tone(buzzerPin, NOTE_C4, 100); //TODO: Test this
        }
        
        //Sleep for 15 seconds... (That's a long time to sleep, so let's keep an eye on the button while we're at it)
        for (int i = 0; i < 30; i++) {
          delay(500); //Sleep for 0.5 seconds
          if (touchChanged == 1) handleTouch(); //Handle the touch event
          if (shutDownEVERYTHING == 1) break;
        }
      }
      
      //TODO: Do something else? 
}

//***************************
// doRemoteControl: Activate the "Remote Control" screen
//
void doRemoteControl(void) {
/*      char incomingValue = 0;  //Store the incoming character
      char remoteMode = 0;       //Store the current 'mode' we are in
      char remoteValue[25];      //Store the mode's values //TODO: Could probably shrink this array size a bit
      char currArrNum = 0;       //Store the current array value we are going to write to
      char okToProcess = 0;      // 0/1 Flag: Is it okay to process the data? (IE did we receive a "\n"?)
      char startByte = 0;        // 0/1 Flag: Have we received the start byte ("~")
      
      currArrNum = 0;
      okToProcess = 0;
      
      delay(100);
      
      char* lines[] = {       "\0",      "\0",      "\0",      "\0",      "\0",      "'Cancel' to Exit\0"    };
      numScreenRows = sizeof(lines)/sizeof(int);
      createScreen(lines, 4, "Remote Control");  //Create a basic screen with the lines from lines[], screenId 2 (Remote Control)
      
      delay(100);
      
//      Serial.println("Ready for remote control commands.  Format: <mode> <val1>[,<val2>,<val3>,<val4>]");
      
      //Remote control modes:
      //--------------------
      // "m" / "M": Motor Control   (Motor 0-1, Direction 0-2, Speed 0-255)
      // "s" / "S": Sensor read     (Sensor: s=Sound, a=Accel, d=Distance...) //TODO: add more?
      // "p" / "P": Play sound      (Sound 0-3)
      // "d" / "D": Display         (Line 0-6, Message (<=18 chars)) [Line 7 is reserved for "Cancel" message]
      // "l" / "L": chainable LEDs  (LED 0-n, Red 0-255, Blue 0-255, Green 0-255)
      
//      Serial.flush();
      
      while (shutDownEVERYTHING == 0) { //As long as we haven't been cancelled
        
          delay(500);
        
        if (Serial.available() > 0) {   //If we have stuff to do, let's do it.
          incomingValue = 0;
          incomingValue = Serial.read();
          
          delay(250);
                  setXY(2,0);
                  sendChar(incomingValue);
          
//          if (incomingValue == 0x0D || incomingValue == 0x0A || incomingValue == 0x7E) { //CR or LF or ~ (tilde so we can do a line feed in the Arduino serial monitor)
          if (incomingValue == 0x7E) { //CR or LF or ~ (tilde so we can do a line feed in the Arduino serial monitor)
//            Serial.println("Carriage Return");

                  setXY(2,0);
                  sendStr("1");
            
            if (remoteMode > 0) {       //If we received a mode already, we're in the process
              okToProcess = 1;          //Set the "Process" flag so that the remoteMode and remoteValue[] will be processed below.
                  setXY(2,2);
                  sendStr("2");
            } else {                    //Else this is a start byte
              startByte = 1;            //Set the flag
                  setXY(2,4);
                  sendStr("3");
            }
          } else if (remoteMode == 0 && startByte == 1) {      //If this is our first byte
            remoteMode = incomingValue;              // read the incoming byte into the remoteMode variable
                  setXY(2,6);
                  sendStr("4");
/*          } else if (currArrNum <= 25) { //Otherwise, if we still have room for remoteValues
            while (currArrNum <= 25 && Serial.available()) {
              remoteValue[currArrNum] = Serial.read(); // read the incoming byte into the remoteValue variable
              currArrNum++;                            //Increase the counter that stores the argument we'll be writing into next time
            }*//*
          } else { //No room for remoteValues
//            Serial.println("No room for additional remoteValues.");
                  setXY(2,8);
                  sendStr("5");            
            while (currArrNum <= 25 && Serial.available()) {
              remoteValue[currArrNum] = Serial.read(); // read the incoming byte into the remoteValue variable
                  setXY(5,0+currArrNum);
                  sendStr("1");
              currArrNum++;                            //Increase the counter that stores the argument we'll be writing into next time
            }
            
/*            setXY(3,0);
            sendStr("No room for more");
            setXY(4,0);
            sendStr("remoteValues    ");
            
            //Cancel the command
            okToProcess = 0;
            remoteMode = 0;
            currArrNum = 0;*//*
          }
          
          if (okToProcess == 1) {
            char temp[18];    //For storing temporary output from sprintf
                              setXY(2,10);
                  sendStr("6");  
            delay(250);
            
            switch (remoteMode) {
//              case 'd':  // "d" / "D": Display         (Line 0-6, Message (<=18 chars)) [Line 7 is reserved for "Cancel" message]
//              case 'D':
                //Make sure we have enough arguments to fit the row, a comma, and at least one character,
                //and that the first value is a number between 0 and 6 inclusive
/*                if (currArrNum > 2 && remoteValue[0] >= 0x30 && remoteValue[0] <= 0x36) {
                  setXY(remoteValue[0] - 48, 0); //Set Y to the row requested (char "0" is 48 characters past char 0, so we fix that)
                  for (int i = 2; i <= currArrNum; i++) {
                    sendChar(remoteValue[i]); //Send the rest of the string
                  }
                }*//*
                break;
              case 'h':
              case 'H':
//                Serial.println("Remote Control Help");
/*                Serial.println("-------------------");
                Serial.println("");
                Serial.println("Chainable RGB LED Control: L"); //TODO
                Serial.println("");
                Serial.println("Motor Control: M<num>,<speed>[,<direction>]");
                Serial.println("                 Num:       0-1");
                Serial.println("                 Speed:     0-255");
                Serial.println("                 Direction: 0=STOP, 1=FWD, 2=REV");
                Serial.println("Play Sound:    P<num>");
                Serial.println("                 Num: 1-3 (or 0 to stop playback)");
                Serial.println("Read Sensor:   S<sensor>");
                Serial.println("                 Sensor: A=Accel, D=Distance, S=Sound");
                Serial.println("");*//*
                break;
              case 'l':  // "l" / "L": chainable LEDs  (LED 0-n, Red 0-255, Blue 0-255, Green 0-255)
              case 'L':    
                  unsigned char ledNum;
                  unsigned char redVal;
                  unsigned char blueVal;
                  unsigned char greenVal;
                  
//                Serial.println("Chainable RGB LEDs"); 
                  setXY(3,0);
                  sendStr("Chainable RGB LEDs");
                  setXY(4,0);
                  //   
                  if (!sscanf(remoteValue,"%d,%d,%d,%d",ledNum, redVal, blueVal, greenVal))
                    sendStr("-> Parse Failed    ");            
                  else {
                    sprintf(temp, "#%3d: %3d %3d %3d", remoteValue[0], redVal, blueVal, greenVal );
                    sendStr(temp);  
                  }

                  break;
              case 'm':  // "m" / "M": Motor Control   (Motor 0-1, Direction 0-2, Speed 0-255)
              case 'M':  
//                Serial.println("Motor Control"); 
                break;
              case 'p':  // "p" / "P": Play sound      (Sound 0-3)
              case 'P':    
//                Serial.println("Play Sound"); 
                break;
              case 's':  // "s" / "S": Sensor read     (Sensor: s=Sound, a=Accel, d=Distance...) //TODO: add more?
              case 'S':   
//                Serial.println("Sensor Read"); 
                break;
              default:
//                Serial.println("Invalid Mode. Valid modes are: D(isplay), L(EDs), M(otor), P(lay sound), S(ensor read).");
                //Serial.println("Type 'H' for help.");
                //Serial.println("> ");
                break;
            }
            //Let's do some cleanup to prepare for the next command
            okToProcess = 0;
            remoteMode = 0;
            currArrNum = 0;
          }
        } 
        
//        delay(150);
        
        //If someone has touched a button since we last looped
        if (touchChanged == 1) {
          //TODO: Send a Bluetooth Message saying we received a button press
//          Serial.println("Got a button press. Going to moon, BRB."); //TODO: Going to assume that I'll set up bluetooth on DIGI 0 and 1, so just plain ol' Serial.print
//          delay(250);
          handleTouch();  //Handle the touch event
//          delay(250);
        }
      }*/
}

//***************************
// doMotorTest: Activate the "Motor Test" screen
//
void doMotorTest(void) {
       char currTest;      //Start out at test "0"/undefined;
      
      currTest++; //Add one to the test counter, we're now on test "1"
      
      if (currTest == 1) { //If this is our first test, load the screen
        char* lines[] = {       "\0",      "\0",      "\0",      "\0",      "\0",      "'Cancel' to Exit\0"    };
        numScreenRows = sizeof(lines)/sizeof(int);
        createScreen(lines, 2, "Motor Test");  //Create a basic screen with the lines from lines[], screenId 1 (Sensor View)
      }
      
      // Update and display data
      while (shutDownEVERYTHING == 0) {
        
       delay(1000);
        
        switch (currTest) {
          case 1:      
            //Let's start testing!
            motorTest(MOTOR_1, MOTOR_FWD, 255);
            break;
          case 2:
            motorTest(MOTOR_1, MOTOR_FWD, 128);
            break;
          case 3:
            motorTest(MOTOR_1, MOTOR_OFF, 0);
            break;
          case 4:
            motorTest(MOTOR_1, MOTOR_REV, 128);
            break;
          case 5:
            motorTest(MOTOR_1, MOTOR_REV, 255);
            break;
          case 6:
            motorTest(MOTOR_1, MOTOR_OFF, 0);
            motorTest(MOTOR_2, MOTOR_FWD, 255);
            break;
          case 7:
            motorTest(MOTOR_2, MOTOR_FWD, 128);
            break;
          case 8:
            motorTest(MOTOR_2, MOTOR_OFF, 0);
            break;
          case 9:
            motorTest(MOTOR_2, MOTOR_REV, 128);
            break;
          case 10:
            motorTest(MOTOR_2, MOTOR_REV, 255);
            break;
          case 11:
            motorTest(MOTOR_2, MOTOR_OFF, 0);
            break;
          default:
            shutDownEVERYTHING = 1;
        }
         
        currTest++;
        
        //If someone has touched a button since we last looped
        if (touchChanged == 1) handleTouch();          
      }
}

//***************************
// doSensorView: Activate the "Sensor View" screen
//
void doSensorView(void) {
    int sumProximity = 0;
    int avgProximity = 0;
    const int proximitySmoothing = 10;
    
    int sumSound = 0;
    int avgSound = 0;
    const int soundSmoothing = 2;
   
        char* lines[] = { 
          " Sensor   | Val\0",
          "----------+-----\0",
          "    PIR   |\0",
          "   Prox   |\0",
          "  Sound   |\0",
          "  Accel X |\0",
          "        Y |\0",
          "        Z |\0"
        };
        numScreenRows = sizeof(lines)/sizeof(int);
  
        createScreen(lines, 1);  //Create a basic screen with the lines from lines[], screenId 1 (Sensor View)
  
        // Update and display data
        while (shutDownEVERYTHING == 0) {      
          char temp[4];
                  
          setXY(2,14);    //PIR
          sendChar(0x30+digitalRead(motionSensor)); 

          //Handle the averaging of proximity values
          if (proximitySmoothing > 1) {
            sumProximity = 0;
            for (char i = 0; i < proximitySmoothing; i++) {
              sumProximity += analogRead(proximitySensor);
              delay(5);
            }
            avgProximity = sumProximity / proximitySmoothing;
          } else {
            avgProximity = analogRead(proximitySensor);
          }
          setXY(3,12);    //Proximity - The proximity Sensor will read from ~30-40 (ambient noise/object too far) to ~645 (5cm away), then from ~645 down to ~320 (0cm away)
          sprintf(temp, "%3d", avgProximity); //SPRINTF() format guide: http://www.cplusplus.com/reference/clibrary/cstdio/sprintf/
          sendStr(temp);

          //Handle the averaging of sound values
          if (soundSmoothing > 1) {
            sumSound = 0;
            for (char i = 0; i < soundSmoothing; i++) {
              sumSound += analogRead(soundSensor);
            }
            avgSound = sumSound / soundSmoothing;
          } else {
            avgSound = analogRead(soundSensor);
          }     
          setXY(4,12);    //Sound
          sprintf(temp, "%3d", avgSound); //SPRINTF() format guide: http://www.cplusplus.com/reference/clibrary/cstdio/sprintf/
          sendStr(temp);
  
/*          setXY(4,11);    //Touch
          //touchStatus = getTouchStatus();
          sprintf(temp, "%d%d%d%d", bitRead(touchStatus, 0),                       
                                    bitRead(touchStatus, 1),
                                    bitRead(touchStatus, 2),        
                                    bitRead(touchStatus, 3)
                 );
          sendStr(temp);*/
          
          //If someone has touched a button since we last checked
          //Checking again here since readAccel() is a bit slow
          //This will increase response time a bit

          if (accelChanged == 1) {
            accelChanged = 0;

            readAccel();

            setXY(5,12);
            sprintf(temp, "%3d", AccelX);
            sendStr(temp);
            setXY(6,12);
            sprintf(temp, "%3d", AccelY);
            sendStr(temp);
            setXY(7,12);
            sprintf(temp, "%3d", AccelZ);
            sendStr(temp);
          }
          
          if (touchChanged == 1) handleTouch();
        } 
}

//***************************
// onAcceleration: Interrupt Request Handler for IRQ 1 (I2C Accelerometer)
//
void onAcceleration(void) {
  accelChanged = 1;
}


//***************************
// onButtonPress: Interrupt Request Handler for IRQ 0 (I2C Touch Sensor)
//      This is activated when someone presses (**or releases!**) a button
//      on the I2C Touch Sensor
//
void onButtonPress(void) {
  touchChanged = 1;
}


//***************************
// getButtonStatus: Find out if buttonNum is currently pressed
// char buttonNum    - The button to find the state of (0-4)
//
char getButtonStatus(char buttonNum) {
  return bitRead(touchStatus, buttonNum);
}


//***************************
// handleTouch: Gets called when a loop discovers
//              that touchChanged == 1
// 
// returns: char   - The button number that was pushed
//
// (PS: Right now I'm not actually using the returned value for anything...
//      I'm actually calling buttonHandler(char) to take care of the buttons)
//
char handleTouch(void) {
    char prevTouchStatus = touchStatus; //Back up the current touch status so we can compare later

    touchStatus = requestI2cData(i2cTouchSensorAddr, 0x00, 1);  //Update the button-press status
    touchChanged = 0;     //Remove the flag that says we got a new button press
      
    //**Now.. What do we do to figure out how to interpret the previous/current Status results?
    
    //Example 1: Let's say someone pushes a button (PUSH)
    //  Previous Status: 0000
    //  New Status:      0010
    //  We Want:         0010
    //  ---------------------
    //  AND returns:     0000
    //  OR returns:      0010
    //  XOR returns:     0010
    
    //Example 2: Let's say someone pushes a second button (PUSH)
    //  Previous Status: 0010
    //  New Status:      0110
    //  We Want:         0100
    //  ---------------------
    //  AND returns:     0010
    //  OR returns:      0110
    //  XOR returns:     0100
    
    //Example 3: Let's say someone lets up on a button (RELEASE)
    //  Previous Status: 0100
    //  New Status:      0000
    //  We Want:         0000
    //  ---------------------
    //  AND returns:     0100       -\
    //  OR returns:      0100       --- No winner for RELEASEs...
    //  XOR returns:     0100       -/
    
    //XOR == A Winnar is You! 
    //  (at least for finding out which button was pressed, not for 
    //   distinguishing between press and release)
    //
    //  Let's XOR the previous and new values to find out which button changed
    //  Then we can see if that bit now == 1 or now == 0 to see if we should act,
    //     since there isn't really a bitwise operation that can tell us that.
    char changedStatus = prevTouchStatus ^ touchStatus;

    char temp[42];
    
/*    sprintf(temp, "Current: %d%d%d%d, Prev: %d%d%d%d, Changed: %d%d%d%d.", bitRead(touchStatus, 0), bitRead(touchStatus, 1), bitRead(touchStatus, 2), bitRead(touchStatus, 3),
                                                           bitRead(prevTouchStatus, 0), bitRead(prevTouchStatus, 1), bitRead(prevTouchStatus, 2), bitRead(prevTouchStatus, 3),
                                                           bitRead(changedStatus, 0), bitRead(changedStatus, 1), bitRead(changedStatus, 2), bitRead(changedStatus, 3)
           );

    Serial.println(temp);
    delay(50);    */

    for (int i=0;i<4;i++) {
      //If the button has changed and is currently pressed...
      if ( bitRead(changedStatus, i) == 1 && bitRead(touchStatus, i) == 1 ) { 
        vibratingMotor_buzz(vibratingMotorBuzz); //Give the user some haptic feedback
        buttonHandler(i);
//        return i;
      }
    }
}


//***************************
// loop: We're not really doing anything else, so watch for major changes,
//         do some housekeeping, etc.
//
void loop()
{
  if (touchChanged == 1) { //Someone pressed a button since we looped last!
    handleTouch(); //Figure out what happened.
  }
  
  if (accelChanged == 1) { //We moved a lot since our last loop
    accelChanged = 0;
    readAccel();
//    handleAccel(); //Figure out what happened.
  }
}

